package ovh.gasser.newshapes.ui.html;

import ovh.gasser.newshapes.ShapeVisitor;
import ovh.gasser.newshapes.attributes.ColorAttributes;
import ovh.gasser.newshapes.shapes.SCollection;
import ovh.gasser.newshapes.shapes.SRectangle;

import java.awt.*;
import java.io.PrintWriter;

public class HTMLDraftman implements ShapeVisitor {

    private static final String HEADER_TEMPLATE = "<!DOCTYPE html>" +
            "<html lang=\"fr\">" +
            "<head>" +
            "   <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />" +
            "   <link rel=\"stylesheet\" href=\"style.css\" />" +
            "   <title>Reactive shapes HTML</title>" +
            "</head>" +
            "<body>";

    private static final String FOOTER_TEMPLATE = "</body>";

    private PrintWriter htmlOutput;
    private PrintWriter cssOutput;

    public HTMLDraftman(PrintWriter htmlOutput, PrintWriter cssOutput) {
        this.htmlOutput = htmlOutput;
        this.cssOutput = cssOutput;
    }

    @Override
    public void visitRectangle(SRectangle rect) {
        htmlOutput.println("<div id=\"rec" + rect.hashCode() + "\"></div>");
        cssOutput.println("#rec" + rect.hashCode() + "{ ");
        cssOutput.println("position:absolute;");
        cssOutput.println("top:" + rect.getBounds().y + "px;");
        cssOutput.println("left:" + rect.getBounds().x + "px;");
        cssOutput.println("width:" + rect.getBounds().width + "px;");
        cssOutput.println("height:" + rect.getBounds().height + "px;");
        cssOutput.println(this.attributesToCss(rect) + " }");
    }

    private String attributesToCss(SRectangle rect) {
        ColorAttributes attrs = (ColorAttributes) rect.getAttributes(ColorAttributes.ID);
        String strokedColor = "#ffffff";
        String filledColor = "#ffffff";

        if (attrs != null && attrs.filledColor != null){
            filledColor = formatCSSColor(attrs.filledColor);
        }
        if (attrs != null && attrs.strokedColor != null){
            strokedColor = formatCSSColor(attrs.strokedColor);
        }
        if(attrs != null && attrs.stroked && attrs.filled){
            return "background: " + filledColor + ";border:1px solid " + strokedColor + ";";
        }
        if (attrs != null && attrs.stroked){
            return "background:#ffffff;border:1px solid "+strokedColor+";";
        }
        if (attrs != null && attrs.filled) {
            return "background: "+filledColor+";";
        }

        return "";
    }

    private String formatCSSColor(Color col) {
        final int r = col.getRed();
        final int g = col.getGreen();
        final int b = col.getBlue();
        return String.format("#%02x%02x%02x", r, g, b);
    }

    @Override
    public void visitCollection(SCollection collection) {
        collection.stream().forEach(shape -> shape.accept(this));
    }
    
    public void generateHTML(SCollection model) {
        htmlOutput.println(HEADER_TEMPLATE);
        visitCollection(model);
        htmlOutput.println(FOOTER_TEMPLATE);
    }
}
